#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <ctype.h>

#define CONF_FILE "/etc/wcal.conf"
#define ADDRESS "127.0.0.1"
#define PORT 13134

#define NENV    250				/* max number of environment variables */
char *newenv[NENV];
char *nullp = NULL;

extern char **environ;
extern errno;
extern char *sys_errlist[];
char *nvmatch ();

char *password;
char *organization;
char *database_directory;
char *user_id;
char *user_fullname;
char *my_url;
int port;

void print_error (const char *message)
{
	if (! message)
		return;
	printf ("<HTML><BODY><H2>%s</H2></BODY></HTML>\n", message);
	return;
}

void doit (void)
{
	int s;
	char buf[8192];
	char curwd[1024];
	char *basename;
	int readlen;
	char **p = newenv;
	struct sockaddr_in sin;


	bzero (&sin, sizeof (sin));
	sin.sin_family = AF_INET;
	sin.sin_addr.s_addr = inet_addr (ADDRESS);
	sin.sin_port = htons (PORT);
	s = socket (AF_INET, SOCK_STREAM, 0);
	if (connect (s, (struct sockaddr *) &sin, sizeof (sin)) < 0) {
		perror ("connect");
		exit (1);
	}
	while (*p != NULL) {
		write (s, *p, strlen (*p));
		write (s, "\n", 1);
		(void) *p++;
	}

	if (! getcwd (curwd, (size_t) 1024)) {
		perror ("getcwd");
		exit (1);
	}
	basename = strrchr (curwd, '/');
	basename++;

	write (s, "__password=", strlen ("__password="));
	write (s, password, strlen (password)); write (s, "\n", 1);

	write (s, "__user=", strlen ("__user="));
	write (s, basename, strlen (basename)); write (s, "\n", 1);

	write (s, "\n", 1);
	while ((readlen = read (s, buf, 8192)) != 0) {
		write (1, buf, readlen);
	}

	close (s);
	return;
}

void addname (char *arg)
{
	register char **p;

	for (p = newenv; *p != NULL; p++) {
		if (p >= &newenv[NENV - 1]) {
			(void) fputs ("too many values in environment\n",
						  stderr);
		}
		if (nvmatch (arg, *p) != NULL)
			break;
	}
	*p = arg;
	return;
}



char * nvmatch (char *s1, char *s2)
{

	while (*s1 == *s2++)
		if (*s1++ == '=')
			return (s2);
	if (*s1 == '\0' && *(s2 - 1) == '=')
		return (s2);
	return (NULL);
}

char *remove_leading_trailing_space (char *s)
{
	char *start;
	while (*s == ' ')
		s++;
	start = s;
	if (! *s)
		return start;
	while (*s)
		s++;
	s--;
	while (*s == ' ')
		s--;
	*(s+1) = '\000';
	return start;
}

int get_conf_line (FILE *file, char **key, char **value)
{
	static char buffer[10240];
	char *lkey, *lvalue;
	int i;
	
	if (! fgets (buffer, 10240, file)) {
		return (0);
	}
	for (i = 0; buffer[i] != '\000'; i++) {
		if (buffer[i] == '\t' || buffer[i] == '\n') {
			buffer[i] = ' ';
		}
	}
	lkey = buffer;
	lvalue = strchr (buffer, '=');
	if (lvalue == NULL) {
		lvalue = "";
	} else {
		*lvalue = '\000';
		lvalue++;
	}
	lkey = remove_leading_trailing_space (lkey);
	for (i = 0; lkey[i] != '\000'; i++) {
		lkey[i] = tolower (lkey[i]);
	}
	lvalue = remove_leading_trailing_space (lvalue);
	*key = lkey;
	*value = lvalue;
	return (1);
}

/*
 * Read the configuration file. Pays attention to the syntax only for the most part.
 * wcald will then decide if most of the data makes sense, for instance,
 * if the appropriate files refered to exist etc.
 */
void read_config ()
{
	FILE *file;
	char *key, *value;
	
	file = fopen (CONF_FILE, "r");
	if (! file) {
		print_error ("can't open " CONF_FILE " for reading");
		exit (1);
	}
	password = NULL;
	while (get_conf_line (file, &key, &value)) {
		if (strcmp (key, "password") == 0) {
			password = strdup (value);
			break;
		}
	}
	if (! password) {
		print_error ("can't find password from configuration file");
		exit (1);
	}
	fclose (file);

	return;
}

int main (int argc, char **argv, char **envp)
{
	read_config ();

	/* Copy environment into newenv */
	for (; *envp != NULL; envp++) {
		if (strchr (*envp, '=') != *envp)
			addname (*envp);
	}

	doit ();
	return 0;
}
